/*
  This file is a part of KMC software distributed under GNU GPL 3 licence.
  The homepage of the KMC project is http://sun.aei.polsl.pl/kmc

  Authors: Sebastian Deorowicz and Agnieszka Debudaj-Grabysz

  Version: 1.0
  Date   : 2014-02-17
*/

#ifndef _KB_STORER_H
#define _KB_STORER_H

#include "defs.h"
#include "params.h"
#include "kmer.h"
#include "radix.h"
#include <string>
#include <algorithm>
#include <numeric>
#include <array>
#include <tuple>
#include <stdio.h>

using namespace std;

//************************************************************************************************************
// CKmerBinStorer - storer of bins of k-mers
//************************************************************************************************************
class CKmerBinStorer {
	CMemoryMonitor *mm;

	vector<string> working_directories;
	int n_bins;
	CBinPartQueue *q_part;
	CBinPackageQueue *q_package;
	CBinDesc *bd;
	uint64 buffer_size_bytes;
	uint64 max_mem_buffer;
	uint64 max_mem_single_package;

	FILE** files;
	uint32 *f_disks;
	uint64 *buf_sizes;
	uint64 max_buf_size;
	uint32 max_buf_size_id;

	typedef list<tuple<uchar *, uint32, uint32>> elem_t; 
	elem_t** buffer;

	void Release();
	string GetName(int n, uint32 &c_disk);
	void CheckBuffer();
	void ReleaseBuffer();
	void PutBinToCompact(uint32 n);

public:
	CKmerBinStorer(CKMCParams &Params, CKMCQueues &Queues);
	~CKmerBinStorer();

	bool OpenFiles();
	void ProcessQueue();
};

//************************************************************************************************************
// CWKmerBinStorer - wrapper for multithreading purposes
//************************************************************************************************************
class CWKmerBinStorer {
	CKmerBinStorer *kbs;

public:
	CWKmerBinStorer(CKMCParams &Params, CKMCQueues &Queues);
	~CWKmerBinStorer();

	void operator()();
};

#endif

// ***** EOF
